/***************************************************************************
 *   Copyright (C) 2006 by LZ   *
 *      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include "hv.h"
#include <qtango.h>
#include <QColor>
#include <QMessageBox>
#include <twidgetgroup.h>

hv::hv(QWidget *parent) : QDialog(parent)
{
	ui.setupUi(this);

	// static TWidgetGroup hv_leds(this);
	// static TWidgetGroup hv_ramps;
	vector <QString> hv_labs = explode(',', "FuG enabled,Ready (Not busy),Switching ON,Switching OFF,Ramping UP,Ramping Down,Ramping to low power,Making a new ramp,Aborting,OFF,Limited Power,Standby,Low Power,Full Power");
	// fill_frame(hv_labs, hv_leds, ui.frame_StateFlags);
	ui.mod_title->setText(QString("Mod %1 HV").arg(qApp->arguments().at(1)));
	if (qApp->arguments()[1] == "1") {
		try {
			hvServer = new Tango::DeviceProxy("p/mod/hv_p1");
		}
		catch (...) {	
			sleep(1);
			try {
				hvServer = new Tango::DeviceProxy("p/mod/hv_p1");
			}
			catch (...) {
				QMessageBox::information(0, "cannot connect HV server",
				QString("Cannot connect HV server for modulator "+qApp->arguments()[1]));
				exit(EXIT_FAILURE);
			}
		}
		try {
			rossServer = new Tango::DeviceProxy("p/mod/ross_p1");
		}
		catch (...) {	
			sleep(1);
			try {
				rossServer = new Tango::DeviceProxy("p/mod/ross_p1");
			}
			catch (...) {
				QMessageBox::information(0, "cannot connect HV server",
				QString("Cannot connect HV server for modulator "+qApp->arguments()[1]));
				exit(EXIT_FAILURE);
			}
		}
	}
	else if (qApp->arguments()[1] == "2") {
		try {
			hvServer = new Tango::DeviceProxy("p/mod/hv_p2");
		}
		catch (...) {	
			sleep(1);
			try {
				hvServer = new Tango::DeviceProxy("p/mod/hv_p2");
			}
			catch (...) {
				QMessageBox::information(0, "cannot connect HV server",
				QString("Cannot connect HV server for modulator "+qApp->arguments()[1]));
				exit(EXIT_FAILURE);
			}
		}
		try {
			rossServer = new Tango::DeviceProxy("p/mod/ross_p2");
		}
		catch (...) {	
			sleep(1);
			try {
				rossServer = new Tango::DeviceProxy("p/mod/ross_p2");
			}
			catch (...) {
				QMessageBox::information(0, "cannot connect Ross server",
				QString("Cannot connect Ross server for modulator "+qApp->arguments()[1]));
				exit(EXIT_FAILURE);
			}
		}
	}
	/* else if (qApp->arguments()[1] == "10") {
		try {
			hvServer = new Tango::DeviceProxy("kg10/mod/hv");
		}
		catch (...) {	
			sleep(1);
			try {
				hvServer = new Tango::DeviceProxy("kg10/mod/hv");
			}
			catch (...) {
				QMessageBox::information(0, "cannot connect HV server",
				QString("Cannot connect HV server for modulator "+qApp->arguments()[1]));
				exit(EXIT_FAILURE);
			}
		}
		try {
			rossServer = new Tango::DeviceProxy("f/mod/ross_f10");
		}
		catch (...) {	
			sleep(1);
			try {
				hvServer = new Tango::DeviceProxy("f/mod/ross_f10");
			}
			catch (...) {
				QMessageBox::information(0, "cannot connect Ross server",
				QString("Cannot connect Ross server for modulator "+qApp->arguments()[1]));
				exit(EXIT_FAILURE);
			}
		}
	}*/
	else {
		QMessageBox::information(0, "cannot connect HV server",
		QString("Cannot connect HV server for modulator "+qApp->arguments()[1]));
		exit(EXIT_FAILURE);
	}
	try {
		ramps = hvServer->read_attribute("ListRamp");
	}
	catch (...) {	
		try {
			ramps = hvServer->read_attribute("ListRamp");
		}
		catch (...) {	
				QMessageBox::information(0, "cannot connect Ross server",
				QString("Cannot connect list ramp for modulator "+qApp->arguments()[1]));
				exit(EXIT_FAILURE);
		}
	}
	vector<string> ramp;
	ramps >> ramp;
	for (unsigned i=0; i<ramp.size(); i++) {
		// ui.comboBox->insertItem(i, ramp[i].c_str(), QVariant(Qt::red));
		ui.comboBox->insertItem(i, ramp[i].c_str());
	}
	// ui.comboBox->setCurrentItem(i);
	connect(ui.View_PushButton, SIGNAL(clicked()), this, SLOT(view_ramp()));
	connect(ui.Load_PushButton, SIGNAL(clicked()), this, SLOT(load_ramp()));
	connect(ui.Save_PushButton, SIGNAL(clicked()), this, SLOT(save_ramp()));
	connect(ui.makerampApply, SIGNAL(clicked()), this, SLOT(make_ramp()));
	connect(ui.Off_PushButton, SIGNAL(clicked()), this, SLOT(hv_off()));
	connect(ui.Ground_PushButton, SIGNAL(clicked()), this, SLOT(ross_ground()));
	try {
		timer = new QTimer(this);
		timer->setInterval(1000);
		timer->setSingleShot(false);
		connect(timer, SIGNAL(timeout()), this, SLOT(myrefresh()));
		timer->start();
	}
	catch (const Tango::DevFailed &e) {
		cout << "cannot connect UI" << endl;
	}
	catch (...) {
		cout << "default exception, cannot setup UI" << endl;
	}

	// QString q_cmd;
	// ui.rampDown_ApplyNumeric->setTarget("p/mod/hv_p"+qApp->arguments()[1]+"->RampDown");
	// ui.RampDn_PushButton->setTargets("p/mod/hv_p"+qApp->arguments()[1]+"->RampDown");
}

hv::~hv()
{
	exit(0);
}

void hv::myrefresh()
{
	TVariant v = ui.stateFlags->currentValue();
	QPalette qp;
	qp.setColor(QPalette::Dark, QColor(!v.toBoolVector(true)[1]? "#808080": "#9f9f9f"));
	qp.setColor(QPalette::Button, QColor(!v.toBoolVector(true)[1]? "#808080": "#efefef"));
	qp.setColor(QPalette::WindowText, QColor(!v.toBoolVector(true)[1]? "#a0a0a0": "#000000"));
	qp.setColor(QPalette::Text, QColor(!v.toBoolVector(true)[1]? "#a0a0a0": "#000000"));
	ui.setVlt_ApplyNumeric->setPalette(qp);
}

void hv::hv_off()
{
	try {
		int r = QMessageBox::question(this, "Confirm HV OFF", "Please confirm command: 'OFF'", "OFF", "Cancel");
		if (r == 0) {
			try {
				hvServer->command_inout("Off");
			}
			catch (...) {
				QMessageBox::information(0, "cannot switch HV off",
				QString("Cannot switch HV off "+qApp->arguments()[1]));
			}
		}
	}
	catch (const Tango::DevFailed &e) {
		cout << "Error: hv_off()" << endl;
	}
}

void hv::ross_ground()
{
	try {
		int r = QMessageBox::question(this, "Confirm Ross Ground", "Please confirm command: 'Ground'", "Ground", "Cancel");
		if (r == 0) {
			try {
				hvServer->command_inout("Off");
				rossServer->command_inout("Off");
			}
			catch (...) {
				QMessageBox::information(0, "cannot switch Ross Ground",
				QString("Cannot switch Ross Ground "+qApp->arguments()[1]));
			}
		}
	}
	catch (const Tango::DevFailed &e) {
		cout << "Error: ross_ground()" << endl;
	}
}

void hv::view_ramp()
{
	// static TWidgetGroup hv_ramp;
	Tango::DeviceData rampName;
	Tango::DeviceData rampData;
	Tango::DevString str = CORBA::string_dup(ui.comboBox->currentText().toStdString().c_str());
	rampName << str;
	try {
		rampData = hvServer->command_inout("ReadRamp", rampName);
	}
	catch (Tango::DevFailed &e) {
		Tango::Except::print_exception(e);
	}
	vector<string> ramp;
	rampData >> ramp;
	size_t pos, pos2;
	if ((pos=ramp[0].find("[["))!=string::npos) {
		pos2 = ramp[0].find("]]]");
		ramp[0] = ramp[0].substr(pos+2,pos2-pos-2);
	}
	ui.textEdit->clear();
	std::string s = ramp[0], s2;
	std::string delimiter = "],[";
	std::string delimiter2 = ",";
	pos = 0;
	std::string token, token2;
	int segment = 1, item;
	while ((pos = s.find(delimiter)) != std::string::npos) {
		token = s.substr(0, pos);
    		s2 = token;
		std::cout << s2 << std::endl;
		pos2 = 0;
		item = 0;
		while ((pos2 = s2.find(delimiter2)) != std::string::npos) {
			token2 = "ramp_on["+std::to_string(segment)+"]."+(item? "current": "voltage")+" = "+s2.substr(0, pos2);
			// std::cout << token2 << std::endl;
			ui.textEdit->append(token2.c_str());
			s2.erase(0, pos2 + delimiter2.length());
			item++;
		}
		ui.textEdit->append(("ramp_on["+std::to_string(segment)+"].time = "+s2).c_str());
		ui.textEdit->append(" ");
		s.erase(0, pos + delimiter.length());
		segment++;
	}
	s2 = s;
	std::cout << s2 << std::endl;
	item = 0;
	while ((pos2 = s2.find(delimiter2)) != std::string::npos) {
		token2 = "ramp_on["+std::to_string(segment)+"]."+(item? "current": "voltage")+" = "+s2.substr(0, pos2);
		// std::cout << token2 << std::endl;
		ui.textEdit->append(token2.c_str());
		s2.erase(0, pos2 + delimiter2.length());
		item++;
	}
	ui.textEdit->append(("ramp_on["+std::to_string(segment)+"].time = "+s2).c_str());
	// ["ramp_22.0kV_7Hz_normal_2019",1,0.1,6,5,0.1,20,[[12,0.3,480],[16,0.3,780],[19,0.3,1080],[21,0.3,1200],[22,0.3,1200]]],
	/*for (unsigned i=0; i<ramp.size(); i++) {
		ui.textEdit->append(ramp[i].substr(0,ramp[i].size()-1).c_str());
	}
[
["ramp_22.0kV_7Hz_normal_2019",1,0.1,6,5,0.1,20,[[12,0.3,480],[16,0.3,780],[19,0.3,1080],[21,0.3,1200],[22,0.3,1200]]],
["ramp_23.0kV_7Hz_conditioning_2019",1,0.1,6,5,0.1,20,[[10,0.3,480],[16,0.3,780],[19,0.3,1080],[23,0.3,3600]]],
["ramp_21.0kV_7Hz_special_2019",1,0.1,6,5,0.1,20,[[12,0.3,480],[16,0.3,780],[19,0.3,1080],[21,0.3,300]]]
]
*/	
}

void hv::load_ramp()
{
	// static TWidgetGroup hv_ramp;
	Tango::DeviceData rampName;
	Tango::DeviceData rampData;
	Tango::DevString str = CORBA::string_dup(ui.comboBox->currentText().toStdString().c_str());
	rampName << str;
	try {
		hvServer->command_inout("LoadRamp", rampName);
	}
	catch (Tango::DevFailed &e) {
		Tango::Except::print_exception(e);
	}
}

void hv::save_ramp()
{
	// static TWidgetGroup hv_ramp;
	Tango::DeviceData rampName;
	Tango::DeviceData rampData;
	Tango::DevVarStringArray	*argout  = new Tango::DevVarStringArray();
	QStringList ramplist = ui.textEdit->toPlainText().split("\n");
	argout->length(ramplist.size()+1);
	(*argout)[0] = CORBA::string_dup(ui.comboBox->currentText().toStdString().c_str());
	for (int i=0; i<ramplist.size(); i++) {
		(*argout)[i+1] = CORBA::string_dup((ramplist[i].toStdString()+"\n").c_str());
		cout << ramplist[i].toStdString() << endl;
	}	
	rampData << argout;
	try {
		hvServer->command_inout("SaveRamp", rampData);
	}
	catch (Tango::DevFailed &e) {
		Tango::Except::print_exception(e);
	}
	return;
}

void hv::make_ramp()
{
	// static TWidgetGroup hv_ramp;
	Tango::DeviceData rampData;
	Tango::DevVarDoubleArray *argout  = new Tango::DevVarDoubleArray();
	argout->length(3);
	(*argout)[0] = ui.makerampV->value();
	(*argout)[1] = ui.makerampI->value();
	(*argout)[2] = ui.makerampT->value();
	cout << ui.makerampV->value() << endl;
	rampData << argout;
	try {
		hvServer->command_inout("MakeRamp", rampData);
	}
	catch (Tango::DevFailed &e) {
		// cout << "Tango::DevFailed " << e.errors[0].reason << endl;
		QMessageBox::information(0, "ERROR", QString(e.errors[0].reason));
		Tango::Except::print_exception(e);
	}
	catch (...) {
		cout << "ERROR: operation failed" << endl;
	}
	/*
	Tango::DeviceData rampName;
	QStringList ramplist = ui.textEdit->toPlainText().split("\n");
	for (unsigned i=0; i<ramplist.size(); i++) {
		(*argout)[i+1] = CORBA::string_dup((ramplist[i].toStdString()+"\n").c_str());
		cout << ramplist[i].toStdString() << endl;
	}	
	*/
	return;
}

vector <QString> explode(const char separator, const char* all)
{
	unsigned i, j;
	vector <QString> v;
	char celem[50];
	v.resize(0);

	for(i=0; i < strlen(all); i++)
	{
		j = 0;
		while(all[i] != separator && i < strlen(all) )
		{
			celem[j] = all[i];
			i++;
			j++;
		}
		celem[j] = '\0';
		v.push_back(QString(celem) );
	}
	return v;
}
